@@ -1,21 +1,25 @@
+import logging
 import os
 import sys
-import tempfile
-import threading
-from contextlib import contextmanager
+from typing import Optional
 
 import click
 from dagster import check
-from dagster.cli.workspace import Workspace, get_workspace_from_kwargs, workspace_target_argument
+from dagster.cli.utils import get_instance_for_service
+from dagster.cli.workspace import (
+    get_workspace_process_context_from_kwargs,
+    workspace_target_argument,
+)
 from dagster.cli.workspace.cli_target import WORKSPACE_TARGET_WARNING
-from dagster.core.instance import DagsterInstance, is_dagster_home_set
 from dagster.core.telemetry import START_DAGIT_WEBSERVER, log_action
+from dagster.core.telemetry_upload import uploading_logging_thread
+from dagster.core.workspace import WorkspaceProcessContext
 from dagster.utils import DEFAULT_WORKSPACE_YAML_FILENAME
+from dagster.utils.log import configure_loggers
 from gevent import pywsgi
 from geventwebsocket.handler import WebSocketHandler
 
-from .app import create_app_from_workspace
-from .telemetry import upload_logs
+from .app import create_app_from_workspace_process_context
 from .version import __version__
 
 
@@ -32,7 +36,7 @@ DEFAULT_DB_STATEMENT_TIMEOUT = 5000  # 5 sec
 @click.command(
     name="ui",
     help=(
-        "Run dagit. Loads a repository or pipeline.\n\n{warning}".format(
+        "Run dagit. Loads a repository or pipeline/job.\n\n{warning}".format(
             warning=WORKSPACE_TARGET_WARNING
         )
         + (
@@ -81,8 +85,19 @@ DEFAULT_DB_STATEMENT_TIMEOUT = 5000  # 5 sec
     type=click.INT,
     show_default=True,
 )
+@click.option(
+    "--read-only",
+    help="Start Dagit in read-only mode, where all mutations such as launching runs and "
+    "turning schedules on/off are turned off.",
+    is_flag=True,
+)
+@click.option(
+    "--suppress-warnings",
+    help="Filter all warnings when hosting Dagit.",
+    is_flag=True,
+)
 @click.version_option(version=__version__, prog_name="dagit")
-def ui(host, port, path_prefix, db_statement_timeout, **kwargs):
+def ui(host, port, path_prefix, db_statement_timeout, read_only, suppress_warnings, **kwargs):
     # add the path for the cwd so imports in dynamically loaded code work correctly
     sys.path.append(os.getcwd())
 
@@ -92,65 +107,74 @@ def ui(host, port, path_prefix, db_statement_timeout, **kwargs):
     else:
         port_lookup = False
 
-    host_dagit_ui(host, port, path_prefix, db_statement_timeout, port_lookup, **kwargs)
+    host_dagit_ui(
+        host,
+        port,
+        path_prefix,
+        db_statement_timeout,
+        port_lookup,
+        read_only,
+        suppress_warnings,
+        **kwargs,
+    )
 
 
-@contextmanager
-def _get_instance():
-    if is_dagster_home_set():
-        with DagsterInstance.get() as instance:
-            yield instance
-    else:
-        with tempfile.TemporaryDirectory() as tempdir:
-            click.echo(
-                f"Using temporary directory {tempdir} for storage. This will be removed when dagit exits.\n"
-                "To persist information across sessions, set the environment variable DAGSTER_HOME to a directory to use.\n"
-            )
-            with DagsterInstance.local_temp(tempdir) as instance:
-                yield instance
+def host_dagit_ui(
+    host,
+    port,
+    path_prefix,
+    db_statement_timeout,
+    port_lookup=True,
+    read_only=False,
+    suppress_warnings=False,
+    **kwargs,
+):
+    if suppress_warnings:
+        os.environ["PYTHONWARNINGS"] = "ignore"
 
-
-def host_dagit_ui(host, port, path_prefix, db_statement_timeout, port_lookup=True, **kwargs):
-
-    with _get_instance() as instance:
+    with get_instance_for_service("dagit") as instance:
         # Allow the instance components to change behavior in the context of a long running server process
         instance.optimize_for_dagit(db_statement_timeout)
 
-        with get_workspace_from_kwargs(kwargs) as workspace:
-            if not workspace:
-                raise Exception("Unable to load workspace with cli_args: {}".format(kwargs))
-
-            host_dagit_ui_with_workspace(instance, workspace, host, port, path_prefix, port_lookup)
+        with get_workspace_process_context_from_kwargs(
+            instance,
+            version=__version__,
+            read_only=read_only,
+            kwargs=kwargs,
+        ) as workspace_process_context:
+            host_dagit_ui_with_workspace_process_context(
+                workspace_process_context, host, port, path_prefix, port_lookup
+            )
 
 
-def host_dagit_ui_with_workspace(instance, workspace, host, port, path_prefix, port_lookup=True):
-    check.inst_param(instance, "instance", DagsterInstance)
-    check.inst_param(workspace, "workspace", Workspace)
-
-    app = create_app_from_workspace(workspace, instance, path_prefix)
-
-    start_server(instance, host, port, path_prefix, app, port_lookup)
-
-
-@contextmanager
-def uploading_logging_thread():
-    stop_event = threading.Event()
-    logging_thread = threading.Thread(
-        target=upload_logs, args=([stop_event]), name="telemetry-upload"
+def host_dagit_ui_with_workspace_process_context(
+    workspace_process_context: WorkspaceProcessContext,
+    host: Optional[str],
+    port: int,
+    path_prefix: str,
+    port_lookup: bool = True,
+):
+    check.inst_param(
+        workspace_process_context, "workspace_process_context", WorkspaceProcessContext
     )
-    try:
-        logging_thread.start()
-        yield
-    finally:
-        stop_event.set()
-        logging_thread.join()
+    check.opt_str_param(host, "host")
+    check.int_param(port, "port")
+    check.str_param(path_prefix, "path_prefix")
+    check.bool_param(port_lookup, "port_lookup")
+
+    app = create_app_from_workspace_process_context(workspace_process_context, path_prefix)
+
+    start_server(workspace_process_context.instance, host, port, path_prefix, app, port_lookup)
 
 
 def start_server(instance, host, port, path_prefix, app, port_lookup, port_lookup_attempts=0):
     server = pywsgi.WSGIServer((host, port), app, handler_class=WebSocketHandler)
 
-    click.echo(
-        "Serving on http://{host}:{port}{path_prefix} in process {pid}".format(
+    configure_loggers()
+    logger = logging.getLogger("dagit")
+
+    logger.info(
+        "Serving dagit on http://{host}:{port}{path_prefix} in process {pid}".format(
             host=host, port=port, path_prefix=path_prefix, pid=os.getpid()
         )
     )
